/*==============================================================================


       eg_13 - piezo effects

               Copyright  2001 Pankhurst Algorithmics - All Rights Reserved
               visit www.ebmDevMag.com for permissions/restrictions on use

==============================================================================*/
#include "gui.h"
#include "gui_pkg.h"

extern "C"
{
#include "piezo.h"
#include "ereader_hostio.h"
#include "sys.h"
#include "timer.h"
}

#include "eg_13_form.h"
/*==============================================================================

       compiler directive - for some functions there is no emulator function
       available, so we have to provide an alternative - comment or uncomment
       the following #define depending on destination:

==============================================================================*/
//#define USING_EMULATOR
/*==============================================================================

       CStringList

==============================================================================*/
class CStringList : public CList
{

  // basic override of CList for display of strings, one per line

  private:

    char **m_entry; // pointer to array of strings

    CWindow *m_window;
    int m_rowHeight;

    U16 GetRowHeight(S32 row);
    void DrawRow(RECT *rect, S32 row);

  public:

    CStringList(int id,int dx,int dy,char **pointerArray,int numRows);

};
//------------------------------------------------------------------------------
CStringList::CStringList(int id,int dx,int dy,char **pointerArray,int numRows)
 : CList(id,dx,dy,numRows,0),
   m_entry(pointerArray),
   m_window(0),
   m_rowHeight(0)
{
}
//------------------------------------------------------------------------------
U16 CStringList::GetRowHeight(S32 row)
{
  // rather than calc the row height each time, for speed we save the value the
  // first time in here, as well as the window pointer
  if ( NULL==m_window ) // first time in?
  {
    m_window=GetWindow();
    m_rowHeight=GUI_FontHeight(m_window->GetFont());
  }
  return m_rowHeight;
}
//------------------------------------------------------------------------------
void CStringList::DrawRow(RECT *rect, S32 row)
{
  // given the rectangle describing the row, draw it
  m_window->DrawText(m_entry[row], rect->x, rect->y);
}
/*==============================================================================

       main window code

==============================================================================*/
void CWindow1::Initialize(void)
{
  // set up list of available jingles for display
  char name[99]=""; // temp name holder
  unsigned int memoryNeeded=0; // total memory needed for all names
  m_noOfJingle=piezo_njingles(&memoryNeeded);
  m_jingleName=new char*[m_noOfJingle];
  // read in names sequentially
  for (int i=0;i<m_noOfJingle;i++)
  {
    if ( piezo_name_jingle( i, name, sizeof(name) ) ) // failed?
    {
      m_noOfJingle=i; // change total to reflect failure
      break;
    }
    m_jingleName[i]=new char[strlen(name)+1];
    strcpy(m_jingleName[i],name);
  }
  // add window list to display results
  RECT rect;
  GetUsableRect(&rect); // get interior dimensions to calculate spacing
  AddChild( new CStringList(101,rect.width-10,110,m_jingleName,m_noOfJingle), 5, rect.height-115);
}
//------------------------------------------------------------------------------
CWindow1::~CWindow1(void)
{
  // clean up
  for (int i=0;i<m_noOfJingle;i++)
    delete[] m_jingleName[i];
  delete[] m_jingleName;
}
//------------------------------------------------------------------------------
S32 CWindow1::MsgHandler(MSG_TYPE type,CViewable *from,S32 data)
{
  if ( MSG_ROW_SELECT==type ) // selected row? then we want to play a jingle
  {
    piezo_play_jingle(data,0);
  }
  else if ( MSG_BUTTON_REPEAT==type ) // button press - play keyboard 
  {
    int key=0;
    switch (from->GetID())
    {
      case CWindow1_BUTTON_C:
        key=62;
        break;
      case CWindow1_BUTTON_CS: // C-sharp
        key=58;
        break;
      case CWindow1_BUTTON_D:
        key=55;
        break;
      case CWindow1_BUTTON_DS: // D-sharp
        key=52;
        break;
      case CWindow1_BUTTON_E:
        key=49;
        break;
      case CWindow1_BUTTON_F:
        key=46;
        break;
      case CWindow1_BUTTON_FS: // F-sharp
        key=43;
        break;
      case CWindow1_BUTTON_G:
        key=41;
        break;
      case CWindow1_BUTTON_GS: // G-sharp
        key=38;
        break;
      case CWindow1_BUTTON_A:
        key=36;
        break;
      case CWindow1_BUTTON_AS: // A-sharp
        key=34;
        break;
      case CWindow1_BUTTON_B:
        key=32;
        break;
    }
    // now play selected tone-duration is (info.man)*4^(15-info.exp) microseconds
    // useful info.exp values:
    //   4 -> ~4 sec intervals
    //   5 -> ~1 sec intervals
    //   6 -> ~1/4 sec intervals - and so on...
    struct piezo_tone info;
    info.tone  =key;
    info.man   =2;
    info.exp   =6; // 2 * (1/4) sec for 1/2 second beeps
    info.unused=0; // should always be set to zero
    //
    // 'piezo_play_1tone' will not compile via 'make', so we need to exclude it;
    // however, when performing 'make seb' remember to undefine USING_EMULATOR
#ifndef USING_EMULATOR
    piezo_play_1tone(info,PIEZO_FLAG_NOQUIET);
#endif
    return 1;
  }
  return CWindow::MsgHandler(type,from,data);
}
//------------------------------------------------------------------------------
void CWindow1::Draw(void)
{
  CWindow::Draw();
  RECT rect;
  GetUsableRect(&rect);
  DrawRectFilled(0,0,rect.width,rect.height,COLOR_GRAY47);
  DrawChildren();
}
/*==============================================================================


       program main

==============================================================================*/
static CWindow1 *f_mainWindow;

//------------------------------------------------------------------------------
S32 GUI_main( MSG_TYPE type, CViewable *object, S32 data )
{
  switch (type)
  {
    case MSG_APP_START:
      if (OS_is_present && hostIO_am_I_the_current_task())
        HOSTIO_INLINE_BREAKPOINT();
      f_mainWindow=new CWindow1();
      f_mainWindow->Initialize();
      return 1;
    case MSG_APP_STOP:
      delete f_mainWindow;
      return 1;
    default:
      break;
  }
  return 0;
};
//------------------------------------------------------------------------------

